function [llh,filtmv,llik_c]=kalmanf(sysm,s00,P00,data)
% [LLH,FILTMV]=kalmanf(SYSM,s00,P00,data)
%
% calculates Kalman estimators using reduced form parameters
%
% REDUCED FORM
%   y(t) = SYSM.DD + SYSM.ZZ*s(t) + SYSM.HH*eta(t)
%   s(t) = SYSM.CC + SYSM.TT*s(t-1) + SYSM.RR*vep(t)
%   where
%       eta(t) ~ iid N(0,I)
%       vep(t) ~ iid N(0,I)
%       eta(t) and vep(t) are independent
%
% INPUT
%   SYSM : system matrices
%        .DD: ny by 1
%        .ZZ: ny by ns
%        .HH: ny by neta
%        .CC: ns by 1
%        .TT: ns by ns
%        .RR: ns by vep
%   s00  : initial mean, ns by 1
%   P00  : initial variance, ns by ns
%   DATA : data matrix, T by ny
%
% OUTPUT
%   LLH     : log likelihood, scalar
%   FILTMV  : filtered means and variances
%           .s0       : conditional mean z(t|t), t=0,...,T, ns by T+1
%           .s1       : mean of forecast z(t+1|t), t=0,...,T-1, ns by T
%           .ltrvecP0 : conditional variances of states,
%                       lower triangular part of P(t|t), ns*(ns+1)/2 by T+1
%           .ltrvecP1 : variances of forecasts of states,
%                       lower triangular part of P(t+1|t), ns*(ns+1)/2 by T

% Sungbae An
% Updated: 10/02/2004

% initialize
nobs = size(data,1);
ns   = size(sysm.CC,1);   % dimension of z
ny   = size(sysm.DD,1);      % dimension of y
ntr  = ns*(ns+1)/2;

llik_c   = zeros(nobs,1);       % log likelihood contribution
s0       = zeros(ns,nobs+1);    % E(s_t|t)
s1       = zeros(ns,nobs);      % E(s_{t+1}|t)
ltrvecP0 = zeros(ntr,nobs+1);   % var(s_t|t)
ltrvecP1 = zeros(ntr,nobs);     % var(s_{t+1}|t)

fcst    = zeros(ny,nobs);
fcsterr = zeros(ny,nobs);

s0(:,1)  = s00;
ltrvecP0(:,1) = ltrvec(P00);

% log likelihood by Kalman filter
for i=1:nobs

    s0t = s0(:,i);
    P0t = ltrmat(ltrvecP0(:,i));

    % forecasting
    s1t = sysm.CC + sysm.TT*s0t;
    P1t = sysm.TT*P0t*sysm.TT' + sysm.RR*sysm.RR';
    P1t = symmat(P1t);

    s1(:,i)       = s1t;
    ltrvecP1(:,i) = ltrvec(P1t);

    % forecast error
    y1    = data(i,:)';
    e1    = y1-sysm.DD-sysm.ZZ*s1t;							% forecase error
    f1inv = inv(sysm.ZZ*P1t*sysm.ZZ'+sysm.HH*sysm.HH');     % inverse of variance of forecast error
    K     = P1t*sysm.ZZ'*f1inv;								% Kalman gain matrix

	% log likelihood
    llik_c(i) = -0.5*ny*log(2*pi) + 0.5*log(det(f1inv)) - 0.5*e1'*f1inv*e1;

    % updating
    s0t = s1t + K*e1;
    P0t = P1t - K*sysm.ZZ*P1t;

    s0(:,i+1)   = s0t;
    ltrvecP0(:,i+1) = ltrvec(P0t);

    fcst(1:ny,i) =sysm.DD+sysm.ZZ*s1t;
    fcsterr(1:ny,i) = e1;
    
end

llh = sum(llik_c);


filtmv.s0 = s0;
filtmv.s1 = s1;
filtmv.ltrvecP0=ltrvecP0;
filtmv.ltrvecP1=ltrvecP1;
filtmv.PT = P0t;

filtmv.fcsterr=fcsterr;
filtmv.fcst=fcst;