function [prob, mse, bandwidth, err] = fcst_eval_np(YYpred, YYact, hmax, average_over_horizon, bscale)
%  fcst_eval_np
%  - Nonparametric probability integral transforms and mean squared
%   errors of forecasts under the unconditional and conditional dist.
% 
% NOTE
%  - The conditional dstributions are approximated via a (normal) kernel
%  density estimation.
%  - For simplicity, this only evalutes contemporaneous forecasts under the
%  conditional distributions.
%  - This is direct translation of professor Schorfheide's gauss program
%  - bscale is not global variable here
% 
% Resulting matrix has following formation
%  - hmax by nv*(nv+1) matrix
%  - first nv column: unconditional objects
%  - next nv column: conditional on first variable
%  - next (j+1) column: conditional on j'th variable
%  - rows: horizon
% updates
% err : prediction error (updated 08/09/2012)


nYYpred   = size(YYpred,1);
ny        = size(YYact,2);
prob      = zeros(hmax, ny*(ny+1));
mse       = zeros(hmax, ny*(ny+1));
bandwidth = zeros(hmax, ny);

err       = zeros(hmax, ny*(ny+1));

if average_over_horizon == 1
    YYpred = transform_YY_means(YYpred, ny);
    YYact  = reshape(YYact, 1, []);
    YYact = transform_YY_means(YYact, ny);
    YYact = reshape(YYact, hmax, ny);
end


% forecasting evaluation loop
%  i: fcst horizon
%  j: conditioning variable
for i=1:1:hmax
    
    % We need to select the ith step forecast
    ith_ind = seqa(0,hmax,ny)+i;
    
    % Get unconditional PIT & MSE
    YYi          = YYpred(:, ith_ind);
    prob(i,1:ny) = mean(YYi < ones(nYYpred,1)*YYact(i,:));
    YYihat       = mean(YYi);
    mse(i,1:ny)  = (YYact(i,:) - YYihat).^2;
    err(i,1:ny)  = (YYact(i,:) - YYihat);
    
    
    bandwidth(i,:) = (bscale*std(YYi)*nYYpred^(-0.2));
    
    % Get conditional PIT & MSE, conitioning on each observable in turn
    for j=1:1:ny
        
        % Get Kernels weights for distribution conditional on YYact(i,j)
        weights = kerneln(YYi(:,j)*ones(1,ny), YYact(i,j), ones(nYYpred,1)*bandwidth(i,:));
        
        % normalize wights
        weights = weights ./ (ones(nYYpred,1)*sum(weights));
    
        prob(i, j*ny+1:(j+1)*ny) ...
            = sum((YYi < ones(nYYpred,1)*YYact(i,:)).*weights);
        YYihat = sum(YYi.*weights);
        mse(i, j*ny+1:(j+1)*ny) = (YYact(i,:) - YYihat).^2;
        err(i, j*ny+1:(j+1)*ny) = (YYact(i,:) - YYihat);
    end
end

    
    
    
    
    
    
    